
// smart_assert_cfgarray.h

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_CONFIGS_ARRAY_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_CONFIGS_ARRAY_HPP_INCLUDED


#if _MSC_VER >= 1020
#pragma once
#endif

#include <map>
#include <boost/function.hpp>
#include "smart_assert_alloc.hpp"
#include "smart_assert_ts.hpp"
#include "smart_assert_ctxfunc.hpp"


// Configurations - allows easier dealing with settings
// related to running the SMART_ASSERT library

namespace boost {

namespace smart_assert {

// forward declaration(s)
class smart_assert_configs_array;
inline smart_assert_configs_array & configs_array();


/*
    Configurations
*/
class smart_assert_configs_array {
    typedef boost::function0< void> config_func;
public:

    smart_assert_configs_array() {
    }

    // associates a configuration with a certain string
    void set_config( const char_type * str, config_func f) {
        Private::lock_type locker( m_cs);

        m_configs[ str] = f;
    }

    // runs the configuration associated  with the given string
    void run_config( const char_type * str) const {
        Private::lock_type locker( m_cs);

        // by default, no persistence; however, the config
        // we're currently running, can override this.
        persistence().clear_persistence_stream();

        config_func_collection::const_iterator found = m_configs.find( str);
        if ( found != m_configs.end() ) 
            // run the function
            (found->second)();
        else
            assert_settings().get_dumper()
                << "[SMART_ASSERT logical error] "
                << "configuration " << str << " not found!" << std::endl;
    }

private:
    // the configurations
    typedef std::map< string_type, config_func, std::less< string_type>, Private::smart_assert_alloc< config_func> > config_func_collection;
    config_func_collection m_configs;

    // thread-safe access
    mutable Private::mutex_type m_cs;
};



inline smart_assert_configs_array & configs_array() {
    static smart_assert_configs_array inst;
    return inst;
}


/*
    configurations for running the SMART_ASSERT library
*/
namespace config {


    /*
        recommended base class for all configurations

        Internally keeps:
        - the logger
        - the handlers for warn, debug, error, fatal
        - the persistence setter (the object that sets or
          clears the persistence)
    */
    struct config_base {
        typedef boost::function0< void> persist_func;

        config_base( 
            assert_context_func logger,
            assert_context_func warn_handler  = &default_handle_warn,
            assert_context_func debug_handler = &default_handle_debug,
            assert_context_func error_handler = &default_handle_error,
            assert_context_func fatal_handler = &default_handle_fatal,
            persist_func persister = persist_func() )
            : m_logger( logger),
              m_warn_handler( warn_handler),
              m_debug_handler( debug_handler),
              m_error_handler( error_handler),
              m_fatal_handler( fatal_handler),
              m_persister( persister)
        {}

        void operator() () {
            if ( m_persister)
                // set persistence
                m_persister();
            else
                // no persister object!
                persistence().clear_persistence_stream();

            assert_settings().set_logger( m_logger);

            assert_settings().set_level_handler( 
                lvl_warn,  m_warn_handler);
            assert_settings().set_level_handler( 
                lvl_debug, m_debug_handler);
            assert_settings().set_level_handler( 
                lvl_error,  m_error_handler);
            assert_settings().set_level_handler( 
                lvl_fatal,  m_fatal_handler);                        
        }
    private:
        // logger
        assert_context_func m_logger;

        // handlers
        assert_context_func m_warn_handler;
        assert_context_func m_debug_handler;
        assert_context_func m_error_handler;
        assert_context_func m_fatal_handler;

        // persister
        persist_func m_persister;
    };


    /*
    Configuration:

    Logger:
        - passed at construction

    Handlers:
        warn  : ignored (does nothing)
        debug : ignored (does nothing)
        error : default
        fatal : default

    Persistence
        - none (as a matter a fact, not needed)

    (note: it would be pretty silly to ignore only debug assertions,
     while leaving the default 'warn' handler, which would
     show Warnings to the user)
    */
    struct ignore_debug_warn : public config_base {
        ignore_debug_warn( assert_context_func logger) : 
            config_base( 
                logger, 
                &ignore_assert,
                &ignore_assert) {
            }
    };


    /*
    Configuration:
        (just like ignore_debug_warn with one difference:
         the 'fatal' level is treated like the 'error' level)

    Logger:
        - passed at construction

    Handlers:
        warn  : ignored (does nothing)
        debug : ignored (does nothing)
        error : throws a smart_assert_error (default)
        fatal : throws a smart_assert_error 

    Persistence
        - none (as a matter a fact, not needed)
    */
    struct ignore_debug_warn_ext : public config_base {
        ignore_debug_warn_ext( assert_context_func logger) : 
            config_base( 
                logger, 
                &ignore_assert,
                &ignore_assert,
                &default_handle_error,
                &default_handle_error) {
            }
    };


    /*
    Configuration:
        logs to a file;
        you can set persistence if you like

    Logger:
        - to a file, whose name is set at construction.

    Handlers:
        warn  : default
        debug : you can set your own (at construction), 
                or use the default
        error : default
        fatal : default

    Persistence
        - to a file, whose name you pass at construction
          (if you don't pass anything, no persistence)
    */
    struct log_debug_to_file : public config_base {

        log_debug_to_file( 
                const char * strLogFileName, 
                assert_context_func debug_handler = &default_handle_debug,
                // ... default means NO persistence!
                const char_type * strPersistFile = "") 
            : config_base( 
                assert_context_func().assign( logger_to_file( strLogFileName)),
                &default_handle_warn,
                debug_handler,
                &default_handle_error,
                &default_handle_fatal,
                persist_to_stream( strPersistFile) ) {
        }
    };

} // namespace config



inline void default_initialize( 
        const char_type * strLogFile, 
        const char_type * strPersistFile, 
        const char_type * strDefaultConfig) {

    // ... logger to file
    logger_to_file log( strLogFile);
    // ... EXTended handler
    ask_user_on_assert_ext dbg_handler;
    // ... EXTended handler (Win32 - shows user-friendly dialog)
    ask_user_on_assert_ext_win32 dbg_handler_win32;

    /*
        "ignore"
        
        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : ignores this type of assertion (does nothing)
                  debug : ignores this type of assertion (does nothing)
                  error : default
                  fatal : default
    */
    config::ignore_debug_warn ignore( assert_context_func().assign( log));
    configs_array().set_config( "ignore", ignore);


    /*
        "ignore_ext"
        
        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : ignores this type of assertion (does nothing)
                  debug : ignores this type of assertion (does nothing)
                  error : throws a smart_assert_error (default)
                  fatal : throws a smart_assert_error

                  This way, no assertion will call abort
    */
    config::ignore_debug_warn_ext ignore_ext( assert_context_func().assign( log));
    configs_array().set_config( "ignore_ext", ignore_ext);


    /*
        "dbg"

        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : default
                  debug : EXTENDED handler
                          (asks user, and user has multiple Ignore options:
                           Ignore/Ignore Forever/Ignore All)
                  error : default
                  fatal : default
    */
    config::log_debug_to_file dbg( strLogFile,
        assert_context_func().assign( dbg_handler) );
    configs_array().set_config( "dbg", dbg);


    /*
        "dbg_win"

        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : default
                  debug : EXTENDED handler
                          (asks user, and user has multiple Ignore options:
                           Ignore/Ignore Forever/Ignore All)
                          Unlike the "dbg" setting, this shows 
                          a graphical (user-friendly) dialog.
                  error : default
                  fatal : default    
    */
    config::log_debug_to_file dbg_win( strLogFile,
        assert_context_func().assign( dbg_handler_win32) );
    configs_array().set_config( "dbg_win", dbg_win);


    /*
        "dbg_persist"
        (just like "dbg" + ignored assertions are persisted)
        
        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : default
                  debug : EXTENDED handler
                          (asks user, and user has multiple Ignore options:
                           Ignore/Ignore Forever/Ignore All)
                          Ignore Forever/Ignore All are persisted
                  error : default
                  fatal : default
    */
    config::log_debug_to_file dbg_persist( 
        strLogFile,
        assert_context_func().assign( dbg_handler),
        strPersistFile);
    configs_array().set_config( "dbg_persist", dbg_persist);


    /*
        "dbg_win_persist"
        (just like "dbg_win" + ignored assertions are persisted)
        
        Logger:   logs all failed assertions to strLogFile.
        Handlers: warn  : default
                  debug : EXTENDED handler
                          Asks user showing a user-friendly dialog.
                          Ignore Forever/Ignore All are persisted
                  error : default
                  fatal : default
    */
    config::log_debug_to_file dbg_win_persist( 
        strLogFile,
        assert_context_func().assign( dbg_handler_win32),
        strPersistFile);
    configs_array().set_config( "dbg_win_persist", dbg_win_persist);


    configs_array().run_config( strDefaultConfig);
}



} // namespace smart_assert

} // namespace boost



#endif
