// smart_assert_h_basic.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_HANDLERS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_HANDLERS_HPP_INCLUDED

#if _MSC_VER >= 1020
#pragma once
#endif

//
// default (BASIC) handlers
//
// note: these are basic handlers;
//
// for cool features,
// #include <boost/smart_assert_ext.hpp>
//
// this should be #included only once, and then
// set the handler (somewhere at the beginning of main() )


#include <stdlib.h>
#include <stdexcept>
#include "smart_assert_context.hpp"
#include "smart_assert_util.hpp"
#include "smart_assert_defs.hpp"

namespace boost { 

namespace smart_assert {

#ifdef WIN32
    #define BOOST_BREAK_INTO_DEBUGGER_DEFINED

    inline void break_into_debugger() {
        __asm { int 3 };
    }
#else
// Define your own break_into_debugger() function !!!
#endif




#ifdef BOOST_BREAK_INTO_DEBUGGER_DEFINED
    // always break on assert
    inline void debugbreak_on_assert( const assert_context & context) {
        break_into_debugger();
    }
#endif


    // ignores the assert
    inline void ignore_assert( const assert_context & context) {
    }


    // aborts on assert
    inline void abort_on_assert( const assert_context & context) {
        abort();
    }


    // prints ALL DETAILS of the assertion to console
    inline void print_details_to_console( const assert_context & context) {
        dump_assert_context_detail( context, std::cout);
    }


    // prints a summary of the assertion to console
    inline void print_summary_to_console( const assert_context & context) {
        dump_assert_context_summary( context, std::cout);
    }

    inline void throw_exc_on_assert( const assert_context & context) {
        ostringstream_type out;
        dump_assert_context_summary( context, out);

#ifndef BOOST_NO_EXCEPTIONS
        throw smart_assert_error( out.str());
#else
        // what could we do?
        abort();
#endif
    }

    // basic - doesn't do very much;
    //
    // for cool features, check out smart_assert_h_ext.h !!!
    inline void basic_ask_user_on_assert( const assert_context & context) {
        // show info to the user
        std::cout << "\n" << Private::get_typeof_level( context.get_level() ) << " in ["
            << context.get_context_file()
            << ":" << context.get_context_line() << "]:\n";

        if ( !context.get_level_msg().empty())
            // we have a user-friendly message
            std::cout << context.get_level_msg() << std::endl;
        else
            std::cout << "Expression: " << context.get_expr() << std::endl;

        // read
        string_type strChooseMsg = "\nChoose option: (M)ore Details, (I)gnore, (D)ebug, (A)bort";
        std::cout << strChooseMsg << std::endl;
        bool bContinue = true;
        while ( bContinue) { 
            bContinue = false;
            char_type ch = std::cin.get();
            switch ( ch) {
            case 'm': case 'M':
                // details
                std::cout << std::endl;
                dump_assert_context_detail( context, std::cout);
                bContinue = true;
                break;

            case 'i': case 'I':
                // ignore
                break;

            case 'd': case 'D':
                // retry
#ifdef BOOST_BREAK_INTO_DEBUGGER_DEFINED
                break_into_debugger();
#else
                // don't know how to break...
                std::cout << "Don't know how to break into debugger..." << std::endl;
                bContinue = true;
#endif
                break;

            case 'a': case 'A':
                // abort
                abort();
                break;

            default:
                bContinue = true;
                break;
            };

            if ( !isspace( ch) && bContinue)
                // ignore spaces
                std::cout << strChooseMsg << std::endl;
        };
    }



    // Thanks Darren Cook!
    inline void ask_user_on_assert_console_detailed( const assert_context & context) {
        // show info to the user
        std::cout << std::endl;
        dump_assert_context_detail( context, std::cout);

        // read
        std::cout << "\nChoose option: I - Ignore, D - Debug, A - Abort" << std::endl;
        bool bContinue = true;
        while ( bContinue) {
            bContinue = false;
            char ch = std::cin.get();
            switch ( ch) {
            case 'i': case 'I':
            // ignore
            break;

            case 'd': case 'D':
            // retry
    #ifdef BOOST_BREAK_INTO_DEBUGGER_DEFINED
            break_into_debugger();
    #else
            // don't know how to break...
            std::cout << "Don't know how to break into debugger..." << std::endl;
            bContinue = true;
    #endif
            break;

            case 'a': case 'A':
            // abort
            abort();
            break;

            default:
            bContinue = true;
            break;
            }

            if ( !isspace( ch) && bContinue)
                // ignore spaces
                std::cout << "\nChoose option: I - Ignore, D - Debug, A - Abort" << std::endl;
        }
    };




    inline void default_handle_warn( const assert_context & context) {
        print_summary_to_console( context);
    }

    inline void default_handle_debug( const assert_context & context) {
        basic_ask_user_on_assert( context);
    }

    inline void default_handle_error( const assert_context & context) {
        throw_exc_on_assert( context);
    }

    inline void default_handle_fatal( const assert_context & context) {
        dump_assert_context_detail( context, std::cout );
        abort_on_assert( context);
    }


} // namespace smart_assert

} // namespace boost




#endif
