// smart_enforce.h

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ENFORCE_HPP_INCLUDED
#define BOOST_INNER_SMART_ENFORCE_HPP_INCLUDED

#if _MSC_VER >= 1020
#pragma once
#endif


#include <string>
#include <sstream>
#include <stdexcept>

namespace boost {

namespace smart_assert {

struct DefaultPredicate
{
  template <class T>
  static bool Wrong(const T& obj)
  {
    return !obj;
  }
};

struct DefaultRaiser
{
    template <class T>
    static void Throw(const T&, const std::string& message, const char* locus)
    {
        // FIXME
        throw std::runtime_error(message + '\n' + locus);
    }
};



template<typename Ref, typename P, typename R>
class Enforcer
{
public:
    Enforcer(Ref t, const char* locus, int nLine) : locus_(P::Wrong(t) ? locus : 0),
        SMART_ENFORCE_A( *this),
        SMART_ENFORCE_B( t)
    {
        if ( locus_)
            std::clog << locus_ << nLine << std::endl;
    }

    template< class type>
    Enforcer& log_current_val( const type & val, const char * str)
    {
        if ( locus_)
        {
            // Here we have time; an exception will be thrown
            std::clog << ", " << str << "=" << val;
        }
        return *this;
    }

    operator Ref() const
    {
        if (locus_) R::Throw(SMART_ENFORCE_B, msg_, locus_);
        return SMART_ENFORCE_B;
    }

public:
    Enforcer& SMART_ENFORCE_A;
    Ref SMART_ENFORCE_B;

private:
    std::string msg_;
    const char* const locus_;
};

template <class P, class R, typename T>
inline Enforcer<const T&, P, R> 
MakeEnforcer(const T& t, const char* locus, int nLine)
{
    return Enforcer<const T&, P, R>(t, locus, nLine);
}

template <class P, class R, typename T>
inline Enforcer<T&, P, R> 
MakeEnforcer(T& t, const char* locus, int nLine)
{
    return Enforcer<T&, P, R>(t, locus, nLine);
}

#define STRINGIZE(expr) STRINGIZE_HELPER(expr)
#define STRINGIZE_HELPER(expr) #expr

// FIXME: create BOOST_CREATE_ENFORCE
#define ENFORCE(exp) \
    boost::smart_assert::MakeEnforcer<boost::smart_assert::DefaultPredicate, boost::smart_assert::DefaultRaiser>((exp), "Expression '" #exp "' failed in '" \
    __FILE__ "', line: ", __LINE__).SMART_ENFORCE_A



#define SMART_ENFORCE_A(x) SMART_ENFORCE_OP(x, B)
#define SMART_ENFORCE_B(x) SMART_ENFORCE_OP(x, A)

#define SMART_ENFORCE_OP(x, next) \
    SMART_ENFORCE_A.log_current_val((x), #x).SMART_ENFORCE_ ## next \
    /**/


} // namespace smart_assert

} // namespace boost

#endif
